@@ -23,7 +23,10 @@ import platform
 
 from distutils.file_util import copy_file
 from distutils.dir_util  import mkpath
-from distutils.dep_util  import newer
+try:
+    from setuptools.modified import newer
+except ImportError:
+    from distutils.dep_util import newer
 
 import distutils.sysconfig
 
@@ -85,7 +88,7 @@ class Configuration(object):
     PKGDIR = 'wx'
     # The name of the top-level package
 
-    SIP_ABI = '12.9'
+    SIP_ABI = '12.14'
     SIP_TRACE = False
 
     # ---------------------------------------------------------------
@@ -158,8 +161,9 @@ class Configuration(object):
             # will probably vary...
             self.WXPLAT = '__WXMSW__'
 
-            if os.environ.get('CPU', None) in ['AMD64', 'X64']:
-                self.VCDLL = 'vc%s_x64_dll' % getVisCVersion()
+            if os.environ.get('CPU', None) in ['AMD64', 'X64', 'ARM64']:
+                arch_str = 'arm64' if os.environ['CPU'] == 'ARM64' else 'x64'
+                self.VCDLL = 'vc%s_%s_dll' % (getVisCVersion(), arch_str)
             else:
                 self.VCDLL = 'vc%s_dll' % getVisCVersion()
 
@@ -277,6 +281,14 @@ class Configuration(object):
                         del LDSHARED[index:index+2]
                     except ValueError:
                         break
+                # also remove any -arch flags and their arg
+                while True:
+                    try:
+                        index = LDSHARED.index('-arch')
+                        # Strip this argument and the next one:
+                        del LDSHARED[index:index+2]
+                    except ValueError:
+                        break
                 LDSHARED = ' '.join(LDSHARED)
                 # Combine with wx's ld command and stash it in the env
                 # where distutils will get it later.
@@ -469,7 +481,7 @@ class Configuration(object):
 
     def build_locale_dir(self, destdir, verbose=1):
         """Build a locale dir under the wxPython package."""
-        moFiles = glob.glob(opj(self.WXDIR, 'locale', '*.mo'))
+        moFiles = sorted(glob.glob(opj(self.WXDIR, 'locale', '*.mo')))
         for src in moFiles:
             lang = os.path.splitext(os.path.basename(src))[0]
             dest = opj(destdir, lang, 'LC_MESSAGES')
@@ -482,18 +494,8 @@ class Configuration(object):
 
     def build_locale_list(self, srcdir):
         # get a list of all files under the srcdir, to be used for install_data
-        if sys.version_info[0] == 2:
-            def walk_helper(lst, dirname, files):
-                for f in files:
-                    filename = opj(dirname, f)
-                    if not os.path.isdir(filename):
-                        lst.append( (dirname, [filename]) )
-            file_list = []
-            os.path.walk(srcdir, walk_helper, file_list)
-            return file_list
-        else:
-            # TODO: Python3 version using os.walk generator
-            return []
+        # TODO: Python3 version using os.walk generator
+        return []
 
 
     def find_data_files(self, srcdir, *wildcards, **kw):
@@ -522,7 +524,7 @@ class Configuration(object):
         else:
             walk_helper((file_list, wildcards),
                         srcdir,
-                        [os.path.basename(f) for f in glob.glob(opj(srcdir, '*'))])
+                        [os.path.basename(f) for f in sorted(glob.glob(opj(srcdir, '*')))])
         return file_list
 
 
@@ -815,7 +817,7 @@ def macFixDependencyInstallName(destdir, prefix, extension, buildDir):
     print("**** macFixDependencyInstallName(%s, %s, %s, %s)" % (destdir, prefix, extension, buildDir))
     pwd = os.getcwd()
     os.chdir(destdir+prefix+'/lib')
-    dylibs = glob.glob('*.dylib')
+    dylibs = sorted(glob.glob('*.dylib'))
     for lib in dylibs:
         #cmd = 'install_name_tool -change %s/lib/%s %s/lib/%s %s' % \
         #      (destdir+prefix,lib,  prefix,lib,  extension)
@@ -907,37 +909,33 @@ def runcmd(cmd, getOutput=False, echoCmd=True, fatal=True, onError=None):
         otherKwArgs = dict(stdout=subprocess.PIPE,
                            stderr=subprocess.STDOUT)
 
-    sp = subprocess.Popen(cmd, shell=True, env=os.environ, **otherKwArgs)
+    with subprocess.Popen(cmd, shell=True, env=os.environ, **otherKwArgs) as sp:
 
-    output = None
-    if getOutput:
-        outputEncoding = 'cp1252' if sys.platform == 'win32' else 'utf-8'
-        output = sp.stdout.read()
-        if sys.version_info > (3,):
-            output = output.decode(outputEncoding)
-        output = output.rstrip()
-
-    rval = sp.wait()
-    if rval:
-        # Failed!
-        #raise subprocess.CalledProcessError(rval, cmd)
-        print("Command '%s' failed with exit code %d." % (cmd, rval))
+        output = None
         if getOutput:
-            print(output)
-        if onError is not None:
-            onError()
-        if fatal:
-            sys.exit(rval)
+            outputEncoding = 'cp1252' if sys.platform == 'win32' else 'utf-8'
+            output = sp.stdout.read()
+            output = output.decode(outputEncoding)
+            output = output.rstrip()
 
-    return output
+        rval = sp.wait()
+        if rval:
+            # Failed!
+            #raise subprocess.CalledProcessError(rval, cmd)
+            print("Command '%s' failed with exit code %d." % (cmd, rval))
+            if getOutput:
+                print(output)
+            if onError is not None:
+                onError()
+            if fatal:
+                sys.exit(rval)
+
+        return output
 
 
 def myExecfile(filename, ns):
-    if sys.version_info < (3,):
-        execfile(filename, ns)
-    else:
-        with open(filename, 'r') as f:
-            exec(f.read(), ns)
+    with open(filename, 'r') as f:
+        exec(f.read(), ns)
 
 
 def textfile_open(filename, mode='rt'):
@@ -947,12 +945,7 @@ def textfile_open(filename, mode='rt'):
     mode parameter must include the 't' to put the stream into text mode.
     """
     assert 't' in mode
-    if sys.version_info < (3,):
-        import codecs
-        mode = mode.replace('t', '')
-        return codecs.open(filename, mode, encoding='utf-8')
-    else:
-        return open(filename, mode, encoding='utf-8')
+    return open(filename, mode, encoding='utf-8')
 
 
 def getSipFiles(names):
@@ -974,7 +967,7 @@ def getVisCVersion():
         raise RuntimeError('getMSVCInfo has not been called yet.')
     # Convert a float like 14.28 to 140, for historical reasons
     # TODO: decide on switching to 142, 143, etc.??
-    ver = str(int(MSVCinfo.vc_ver)) + '0'
+    ver = str(int(MSVCinfo["vc_ver"])) + '0'
     return ver
 
 
@@ -987,7 +980,7 @@ def getExpectedVisCVersion():
     """
     if MSVCinfo is None:
         raise RuntimeError('getMSVCInfo has not been called yet.')
-    py_ver = MSVCinfo.py_ver
+    py_ver = MSVCinfo["py_ver"]
     if py_ver in ((3, 5), (3, 6), (3, 7), (3, 8)):
         min_ver = 14.0
     elif py_ver in ((3, 9), (3, 10)):
@@ -1006,8 +999,6 @@ def getMSVCInfo(PYTHON, arch, set_env=False):
     if MSVCinfo is not None:
         return MSVCinfo
 
-    from attrdict import AttrDict
-
     # Note that it starts with a monkey-patch in setuptools.msvc to
     # workaround this issue: pypa/setuptools#1902
     cmd = \
@@ -1022,13 +1013,13 @@ def getMSVCInfo(PYTHON, arch, set_env=False):
         "print(env)"
     cmd = cmd.format(arch)
     env = eval(runcmd('"%s" -c "%s"' % (PYTHON, cmd), getOutput=True, echoCmd=False))
-    info = AttrDict(env)
+    info = dict(env)
 
     if set_env:
-        os.environ['PATH'] =    info.path
-        os.environ['INCLUDE'] = info.include
-        os.environ['LIB'] =     info.lib
-        os.environ['LIBPATH'] = info.libpath
+        os.environ['PATH'] =    info["path"]
+        os.environ['INCLUDE'] = info["include"]
+        os.environ['LIB'] =     info["lib"]
+        os.environ['LIBPATH'] = info["libpath"]
 
         # We already have everything we need, tell distutils to not go hunting
         # for it all again if it happens to be called.
