@@ -3,7 +3,7 @@
 # Author:      Robin Dunn
 #
 # Created:     2-Sept-2011
-# Copyright:   (c) 2011-2017 by Total Control Software
+# Copyright:   (c) 2011-2020 by Total Control Software
 # License:     wxWindows License
 #---------------------------------------------------------------------------
 
@@ -17,7 +17,12 @@ DOCSTRING = ""
 
 # The classes and/or the basename of the Doxygen XML files to be processed by
 # this script.
-ITEMS  = [ 'wxSVGFileDC' ]
+ITEMS  = [ 'wxSVGFileDC',
+           'wxSVGBitmapHandler',
+           'wxSVGBitmapEmbedHandler',
+           'wxSVGBitmapFileHandler',
+
+           ]
 
 #---------------------------------------------------------------------------
 
