@@ -3,7 +3,7 @@
 # Author:      Robin Dunn
 #
 # Created:     29-Oct-2011
-# Copyright:   (c) 2011-2018 by Total Control Software
+# Copyright:   (c) 2011-2020 by Total Control Software
 # License:     wxWindows License
 #---------------------------------------------------------------------------
 
@@ -39,6 +39,19 @@ def run():
     c.find('Create.label').default = 'wxEmptyString'
     tools.fixWindowClass(c)
 
+    # TODO: The window-label ctor is only available on MSW and GTK so disable
+    # for now. Maybe replace it with a factory function that returns None on
+    # OSX??
+    c.find('wxStaticBox').findOverload('wxWindow *label').ignore()
+    c.find('Create').findOverload('wxWindow *label').ignore()
+
+    # "unfix" the 2nd ctor and Create method so the required parameter lists
+    # are different enough for them to be overloaded.
+    #for name in ['wxStaticBox', 'Create']:
+    #    m = c.find(name).findOverload('wxWindow *label')
+    #    m.find('id').default = ''
+
+
     # This is intentionally not documented, but I think it would be handy to
     # use from wxPython.
     meth = MethodDef(
