@@ -33,6 +33,9 @@ enum wxScrollbarVisibility
       so doesn't handle children specially. This is suitable e.g. for
       implementing scrollable controls such as tree or list controls.
 
+    @note
+    See wxScrolled::Create() if you want to use wxScrolled with a custom class.
+
     Starting from version 2.4 of wxWidgets, there are several ways to use a
     ::wxScrolledWindow (and now wxScrolled). In particular, there are
     three ways to set the size of the scrolling area:
@@ -231,8 +234,13 @@ public:
 
     /**
         Creates the window for two-step construction. Derived classes
-        should call or replace this function. See wxScrolled::wxScrolled()
-        for details.
+        should call or replace this function. If it is not replaced,
+        bear in mind that it calls T::Create() through the global function
+        wxCreateScrolled() so if T::Create() has a different signature
+        than wxScrolled::Create() you should implement overloaded
+        wxCreateScrolled() which would call T::Create() in the correct manner.
+
+        @see wxScrolled::wxScrolled() and wxCreateScrolled() for details.
     */
     bool Create(wxWindow* parent, wxWindowID id = -1,
                 const wxPoint& pos = wxDefaultPosition,
@@ -332,6 +340,8 @@ public:
             - wxSHOW_SB_DEFAULT: To restore the default behaviour described
                 above.
 
+        Note that the window must be created before calling this method.
+
         @param horz
             The desired visibility for the horizontal scrollbar.
         @param vert
@@ -537,7 +547,7 @@ public:
     void SetTargetWindow(wxWindow *window);
     wxWindow *GetTargetWindow() const;
 
-    
+
     void SetTargetRect(const wxRect& rect);
     wxRect GetTargetRect() const;
 
@@ -560,19 +570,32 @@ public:
        window.
      */
     void StopAutoScrolling();
-    
+
     /**
        This method can be overridden in a derived class to forbid sending the
        auto scroll events - note that unlike StopAutoScrolling() it doesn't
        stop the timer, so it will be called repeatedly and will typically
        return different values depending on the current mouse position
-    
+
        The base class version just returns true.
     */
     virtual bool SendAutoScrollEvents(wxScrollWinEvent& event) const;
 
-
 protected:
+    /**
+        This method can be overridden in a derived class to prevent scrolling
+        the child window into view automatically when it gets focus.
+
+        The default behaviour is to scroll this window to show its currently
+        focused child automatically, to ensure that the user can interact with
+        it. This is usually helpful, but can be undesirable for some windows,
+        in which case this method can be overridden to return @false for them
+        to prevent any scrolling from taking place when such windows get focus.
+
+        @since 3.1.3
+     */
+    virtual bool ShouldScrollToChildOnFocus(wxWindow* child);
+
     /**
         Function which must be overridden to implement the size available for
         the scroll target for the given size of the main window.
@@ -588,6 +611,32 @@ protected:
     virtual wxSize GetSizeAvailableForScrollTarget(const wxSize& size);
 };
 
+/**
+    Helper function which is called from wxScrolled::Create() to actually create
+    a scrolled window. By default it just passes the call to the base class Create():
+    @code
+    self->Create(parent, winid, pos, size, style, name);
+    @endcode
+
+    You should provide overloaded implementation of this function for the custom
+    base class if this class is created in a different manner, like it is e.g.
+    done for wxControl:
+    @code
+    bool wxCreateScrolled(wxControl* self,
+                          wxWindow *parent, wxWindowID winid,
+                          const wxPoint& pos, const wxSize& size,
+                          long style, const wxString& name)
+    {
+         return self->Create(parent, winid, pos, size, style, wxDefaultValidator, name);
+    }
+    @endcode
+
+    @since 3.1.3
+*/
+bool wxCreateScrolled(T* self,
+                      wxWindow *parent, wxWindowID winid,
+                      const wxPoint& pos, const wxSize& size,
+                      long style, const wxString& name);
 
 /**
     Scrolled window derived from wxPanel.
