@@ -19,9 +19,6 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #include <stdarg.h>
 #include <stdlib.h>
@@ -51,6 +48,7 @@ bool wxListKey::operator==(wxListKeyValue value) const
             wxFAIL_MSG(wxT("bad key type."));
             // let compiler optimize the line above away in release build
             // by not putting return here...
+            wxFALLTHROUGH;
 
         case wxKEY_STRING:
             return *m_key.string == *value.string;
@@ -146,16 +144,6 @@ void wxListBase::Init(wxKeyType keyType)
   m_keyType = keyType;
 }
 
-wxListBase::wxListBase(size_t count, void *elements[])
-{
-  Init();
-
-  for ( size_t n = 0; n < count; n++ )
-  {
-      Append(elements[n]);
-  }
-}
-
 void wxListBase::DoCopy(const wxListBase& list)
 {
     wxASSERT_MSG( !list.m_destroy,
@@ -360,7 +348,7 @@ void wxListBase::DoDeleteNode(wxNodeBase *node)
     // free node's data
     if ( m_keyType == wxKEY_STRING )
     {
-        free(node->m_key.string);
+        wxDELETE(node->m_key.string);
     }
 
     if ( m_destroy )
@@ -511,9 +499,6 @@ void wxListBase::Sort(const wxSortCompareFunction compfunc)
 
     // sort the array
     qsort((void *)objArray,num,sizeof(wxObject *),
-#ifdef __WXWINCE__
-        (int (__cdecl *)(const void *,const void *))
-#endif
         compfunc);
 
     // put the sorted pointers back into the list
@@ -695,22 +680,16 @@ bool wxStringList::Member(const wxChar *s) const
     return false;
 }
 
-#ifdef __WXWINCE__
-extern "C"
-{
-static int __cdecl
-#else
 extern "C"
 {
 static int LINKAGEMODE
-#endif
 
 wx_comparestrings(const void *arg1, const void *arg2)
 {
-  wxChar **s1 = (wxChar **) arg1;
-  wxChar **s2 = (wxChar **) arg2;
+    const wxChar* s1 = *static_cast<wxChar* const*>(arg1);
+    const wxChar* s2 = *static_cast<wxChar* const*>(arg2);
 
-  return wxStrcmp (*s1, *s2);
+    return wxStrcmp(s1, s2);
 }
 
 }   // end of extern "C" (required because of GCC Bug c++/33078
