@@ -18,9 +18,6 @@
 // for compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #if wxUSE_HEADERCTRL
 
@@ -58,6 +55,7 @@ void wxHeaderCtrl::Init()
     m_colBeingReordered = COL_NONE;
     m_dragOffset = 0;
     m_scrollOffset = 0;
+    m_wasSeparatorDClick = false;
 }
 
 bool wxHeaderCtrl::Create(wxWindow *parent,
@@ -73,7 +71,7 @@ bool wxHeaderCtrl::Create(wxWindow *parent,
 
     // tell the system to not paint the background at all to avoid flicker as
     // we paint the entire window area in our OnPaint()
-    SetBackgroundStyle(wxBG_STYLE_CUSTOM);
+    SetBackgroundStyle(wxBG_STYLE_PAINT);
 
     return true;
 }
@@ -93,6 +91,11 @@ void wxHeaderCtrl::DoSetCount(unsigned int count)
 
     m_numColumns = count;
 
+    // don't leave the column index invalid, this would cause a crash later if
+    // it is used from OnMouse()
+    if ( m_hover >= count )
+        m_hover = COL_NONE;
+
     InvalidateBestSize();
     Refresh();
 }
@@ -135,11 +138,9 @@ wxSize wxHeaderCtrl::DoGetBestSize() const
 
     // the vertical size is rather arbitrary but it looks better if we leave
     // some space around the text
-    const wxSize size(IsEmpty() ? wxHeaderCtrlBase::DoGetBestSize().x
-                                : GetColEnd(GetColumnCount() - 1),
-                                height ); // (7*GetCharHeight())/4);
-    CacheBestSize(size);
-    return size;
+    return wxSize(IsEmpty() ? wxHeaderCtrlBase::DoGetBestSize().x
+                            : GetColEnd(GetColumnCount() - 1),
+                  height); // (7*GetCharHeight())/4);
 }
 
 int wxHeaderCtrl::GetColStart(unsigned int idx) const
@@ -166,9 +167,10 @@ int wxHeaderCtrl::GetColEnd(unsigned int idx) const
     return x + GetColumn(idx).GetWidth();
 }
 
-unsigned int wxHeaderCtrl::FindColumnAtPoint(int x, bool *onSeparator) const
+unsigned int wxHeaderCtrl::FindColumnAtPoint(int xPhysical, bool *onSeparator) const
 {
     int pos = 0;
+    int xLogical = xPhysical - m_scrollOffset;
     const unsigned count = GetColumnCount();
     for ( unsigned n = 0; n < count; n++ )
     {
@@ -179,11 +181,12 @@ unsigned int wxHeaderCtrl::FindColumnAtPoint(int x, bool *onSeparator) const
 
         pos += col.GetWidth();
 
+        // TODO: don't hardcode sensitivity
+        const int separatorClickMargin = FromDIP(8);
+
         // if the column is resizable, check if we're approximatively over the
         // line separating it from the next column
-        //
-        // TODO: don't hardcode sensitivity
-        if ( col.IsResizeable() && abs(x - pos) < 8 )
+        if ( col.IsResizeable() && abs(xLogical - pos) < separatorClickMargin )
         {
             if ( onSeparator )
                 *onSeparator = true;
@@ -191,7 +194,7 @@ unsigned int wxHeaderCtrl::FindColumnAtPoint(int x, bool *onSeparator) const
         }
 
         // inside this column?
-        if ( x < pos )
+        if ( xLogical < pos )
         {
             if ( onSeparator )
                 *onSeparator = false;
@@ -204,6 +207,23 @@ unsigned int wxHeaderCtrl::FindColumnAtPoint(int x, bool *onSeparator) const
     return COL_NONE;
 }
 
+unsigned int wxHeaderCtrl::FindColumnClosestToPoint(int xPhysical) const
+{
+    const unsigned int colIndexAtPoint = FindColumnAtPoint(xPhysical);
+
+    // valid column found?
+    if ( colIndexAtPoint != COL_NONE )
+        return colIndexAtPoint;
+
+    // if not, xPhysical must be beyond the rightmost column, so return its
+    // index instead -- if we have it
+    const unsigned int count = GetColumnCount();
+    if ( !count )
+        return COL_NONE;
+
+    return m_colIndices[count - 1];
+}
+
 // ----------------------------------------------------------------------------
 // wxHeaderCtrl repainting
 // ----------------------------------------------------------------------------
@@ -368,7 +388,7 @@ void wxHeaderCtrl::UpdateReorderingMarker(int xPhysical)
 
     // and also a hint indicating where it is going to be inserted if it's
     // dropped now
-    unsigned int col = FindColumnAtPoint(xPhysical);
+    unsigned int col = FindColumnClosestToPoint(xPhysical);
     if ( col != COL_NONE )
     {
         static const int DROP_MARKER_WIDTH = 4;
@@ -410,21 +430,30 @@ bool wxHeaderCtrl::EndReordering(int xPhysical)
 
     ReleaseMouse();
 
-    const int colOld = m_colBeingReordered,
-              colNew = FindColumnAtPoint(xPhysical);
+    const int colOld = m_colBeingReordered;
+    const unsigned colNew = FindColumnClosestToPoint(xPhysical);
 
     m_colBeingReordered = COL_NONE;
 
+    // mouse drag must be longer than min distance m_dragOffset
     if ( xPhysical - GetColStart(colOld) == m_dragOffset )
+    {
         return false;
+    }
 
-    if ( colNew != colOld )
+    // cannot proceed without a valid column index
+    if ( colNew == COL_NONE )
+    {
+        return false;
+    }
+
+    if ( static_cast<int>(colNew) != colOld )
     {
         wxHeaderCtrlEvent event(wxEVT_HEADER_END_REORDER, GetId());
         event.SetEventObject(this);
         event.SetColumn(colOld);
 
-        const unsigned pos = GetColumnPos(FindColumnAtPoint(xPhysical));
+        const unsigned pos = GetColumnPos(colNew);
         event.SetNewOrder(pos);
 
         if ( !GetEventHandler()->ProcessEvent(event) || event.IsAllowed() )
@@ -465,7 +494,7 @@ void wxHeaderCtrl::DoMoveCol(unsigned int idx, unsigned int pos)
 // wxHeaderCtrl event handlers
 // ----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(wxHeaderCtrl, wxHeaderCtrlBase)
+wxBEGIN_EVENT_TABLE(wxHeaderCtrl, wxHeaderCtrlBase)
     EVT_PAINT(wxHeaderCtrl::OnPaint)
 
     EVT_MOUSE_EVENTS(wxHeaderCtrl::OnMouse)
@@ -473,19 +502,15 @@ BEGIN_EVENT_TABLE(wxHeaderCtrl, wxHeaderCtrlBase)
     EVT_MOUSE_CAPTURE_LOST(wxHeaderCtrl::OnCaptureLost)
 
     EVT_KEY_DOWN(wxHeaderCtrl::OnKeyDown)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 void wxHeaderCtrl::OnPaint(wxPaintEvent& WXUNUSED(event))
 {
     int w, h;
     GetClientSize(&w, &h);
 
-#ifdef __WXGTK__
-//    int vw;
-//    GetVirtualSize(&vw, NULL);
-#endif
-
     wxAutoBufferedPaintDC dc(this);
+    dc.Clear();
 
     // account for the horizontal scrollbar offset in the parent window
     dc.SetDeviceOrigin(m_scrollOffset, 0);
@@ -528,13 +553,12 @@ void wxHeaderCtrl::OnPaint(wxPaintEvent& WXUNUSED(event))
 
         wxHeaderButtonParams params;
         params.m_labelText = col.GetTitle();
-        params.m_labelBitmap = col.GetBitmap();
+        params.m_labelBitmap = col.GetBitmapBundle().GetBitmapFor(this);
         params.m_labelAlignment = col.GetAlignment();
 
 #ifdef __WXGTK__
         if (i == count-1 && xpos + colWidth >= w)
         {
-//            colWidth = wxMax( colWidth, vw - xpos );
             state |= wxCONTROL_DIRTY;
         }
 #endif
@@ -585,6 +609,9 @@ void wxHeaderCtrl::OnKeyDown(wxKeyEvent& event)
 
 void wxHeaderCtrl::OnMouse(wxMouseEvent& mevent)
 {
+    const bool wasSeparatorDClick = m_wasSeparatorDClick;
+    m_wasSeparatorDClick = false;
+
     // do this in advance to allow simply returning if we're not interested,
     // we'll undo it if we do handle the event below
     mevent.Skip();
@@ -592,7 +619,6 @@ void wxHeaderCtrl::OnMouse(wxMouseEvent& mevent)
 
     // account for the control displacement
     const int xPhysical = mevent.GetX();
-    const int xLogical = xPhysical - m_scrollOffset;
 
     // first deal with the [continuation of any] dragging operations in
     // progress
@@ -627,7 +653,7 @@ void wxHeaderCtrl::OnMouse(wxMouseEvent& mevent)
     bool onSeparator;
     const unsigned col = mevent.Leaving()
                             ? (onSeparator = false, COL_NONE)
-                            : FindColumnAtPoint(xLogical, &onSeparator);
+                            : FindColumnAtPoint(xPhysical, &onSeparator);
 
 
     // update the highlighted column if it changed
@@ -661,8 +687,11 @@ void wxHeaderCtrl::OnMouse(wxMouseEvent& mevent)
             wxASSERT_MSG( !IsResizing(), "reentering column resize mode?" );
             StartOrContinueResizing(col, xPhysical);
         }
-        else // on column itself
+        // on column itself - both header and column must have the appropriate
+        // flags to allow dragging the column
+        else if ( HasFlag(wxHD_ALLOW_REORDER) && GetColumn(col).IsReorderable() )
         {
+
             // start dragging the column
             wxASSERT_MSG( !IsReordering(), "reentering column move mode?" );
 
@@ -685,8 +714,9 @@ void wxHeaderCtrl::OnMouse(wxMouseEvent& mevent)
                 if ( onSeparator && dblclk )
                 {
                     evtType = wxEVT_HEADER_SEPARATOR_DCLICK;
+                    m_wasSeparatorDClick = true;
                 }
-                else // not double click on separator
+                else if (!wasSeparatorDClick)
                 {
                     evtType = click ? wxEVT_HEADER_CLICK
                                     : wxEVT_HEADER_DCLICK;
