@@ -13,9 +13,6 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #if wxUSE_SASH
 
@@ -36,10 +33,10 @@
 
 wxDEFINE_EVENT( wxEVT_SASH_DRAGGED, wxSashEvent );
 
-IMPLEMENT_DYNAMIC_CLASS(wxSashWindow, wxWindow)
-IMPLEMENT_DYNAMIC_CLASS(wxSashEvent, wxCommandEvent)
+wxIMPLEMENT_DYNAMIC_CLASS(wxSashWindow, wxWindow);
+wxIMPLEMENT_DYNAMIC_CLASS(wxSashEvent, wxCommandEvent);
 
-BEGIN_EVENT_TABLE(wxSashWindow, wxWindow)
+wxBEGIN_EVENT_TABLE(wxSashWindow, wxWindow)
     EVT_PAINT(wxSashWindow::OnPaint)
     EVT_SIZE(wxSashWindow::OnSize)
     EVT_MOUSE_EVENTS(wxSashWindow::OnMouseEvent)
@@ -47,7 +44,7 @@ BEGIN_EVENT_TABLE(wxSashWindow, wxWindow)
     EVT_SET_CURSOR(wxSashWindow::OnSetCursor)
 #endif // __WXMSW__ || __WXMAC__
 
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 bool wxSashWindow::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos,
     const wxSize& size, long style, const wxString& name)
@@ -548,7 +545,6 @@ void wxSashWindow::DrawSashTracker(wxSashEdgePosition edge, int x, int y)
     int w, h;
     GetClientSize(&w, &h);
 
-    wxScreenDC screenDC;
     int x1, y1;
     int x2, y2;
 
@@ -586,18 +582,34 @@ void wxSashWindow::DrawSashTracker(wxSashEdgePosition edge, int x, int y)
     ClientToScreen(&x1, &y1);
     ClientToScreen(&x2, &y2);
 
+#ifdef __WXGTK3__
+    // We need to draw over the parent window, not this one, as we want to
+    // allow the sash go outside of this window.
+    wxWindow* const parent = wxGetTopLevelParent(this);
+    if ( !parent )
+        return;
+
+    wxClientDC dc(parent);
+    parent->ScreenToClient(&x1, &y1);
+    parent->ScreenToClient(&x2, &y2);
+
+    // In the ports with wxGraphicsContext-based wxDC, wxINVERT only works for
+    // inverting the background when using white foreground (note that this
+    // code is not going to work anyhow with wxOSX nor with wxGTK when using
+    // Wayland, as drawing using wxClientDC doesn't work at all there), but
+    // this at least makes it work with wxGTK with X11.
+    wxPen sashTrackerPen(*wxWHITE, 2, wxPENSTYLE_SOLID);
+#else
+    wxScreenDC dc;
+
     wxPen sashTrackerPen(*wxBLACK, 2, wxPENSTYLE_SOLID);
+#endif
 
-    screenDC.SetLogicalFunction(wxINVERT);
-    screenDC.SetPen(sashTrackerPen);
-    screenDC.SetBrush(*wxTRANSPARENT_BRUSH);
+    dc.SetLogicalFunction(wxINVERT);
+    dc.SetPen(sashTrackerPen);
+    dc.SetBrush(*wxTRANSPARENT_BRUSH);
 
-    screenDC.DrawLine(x1, y1, x2, y2);
-
-    screenDC.SetLogicalFunction(wxCOPY);
-
-    screenDC.SetPen(wxNullPen);
-    screenDC.SetBrush(wxNullBrush);
+    dc.DrawLine(x1, y1, x2, y2);
 }
 
 // Position and size subwindows.
