@@ -390,6 +390,7 @@ bool wxSpinCtrlGenericBase::SetBackgroundColour(const wxColour& colour)
 wxBEGIN_EVENT_TABLE(wxSpinCtrlGenericBase, wxSpinCtrlBase)
     EVT_CHAR(wxSpinCtrlGenericBase::OnTextChar)
     EVT_KILL_FOCUS(wxSpinCtrlGenericBase::OnTextLostFocus)
+    EVT_MOUSEWHEEL(wxSpinCtrlGenericBase::OnMouseWheel)
 wxEND_EVENT_TABLE()
 
 void wxSpinCtrlGenericBase::OnSpinButton(wxSpinEvent& event)
@@ -477,6 +478,34 @@ void wxSpinCtrlGenericBase::OnTextChar(wxKeyEvent& event)
         DoSendEvent();
 }
 
+void wxSpinCtrlGenericBase::OnMouseWheel(wxMouseEvent& event)
+{
+    if ( event.GetWheelAxis() != wxMOUSE_WHEEL_VERTICAL || event.GetWheelRotation() == 0 )
+    {
+        event.Skip();
+        return;
+    }
+
+    double value = m_value;
+
+    if( event.GetWheelRotation() > 0 )
+    {
+        value += m_increment;
+    }
+    else if ( event.GetWheelRotation() < 0 )
+    {
+        value -= m_increment;
+    }
+
+    value = AdjustToFitInRange(value);
+
+    SyncSpinToText(SendEvent_None);
+
+    // No need to send event, it was already generated by wxTextCtrl itself.
+    if ( DoSetValue(value, SendEvent_None) )
+        DoSendEvent();
+}
+
 // ----------------------------------------------------------------------------
 // Textctrl functions
 // ----------------------------------------------------------------------------
