@@ -18,9 +18,6 @@
 
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 #if wxUSE_MENUS
 
@@ -183,10 +180,12 @@ protected:
     virtual void DoDraw(wxControlRenderer *renderer);
 
     // event handlers
+    void OnLeftDown(wxMouseEvent& event);
     void OnLeftUp(wxMouseEvent& event);
     void OnMouseMove(wxMouseEvent& event);
     void OnMouseLeave(wxMouseEvent& event);
     void OnKeyDown(wxKeyEvent& event);
+    void OnCaptureLost(wxMouseCaptureLostEvent& event);
 
     // reset the current item and node
     void ResetCurrent();
@@ -208,7 +207,7 @@ protected:
     // show the submenu for this item
     void OpenSubmenu(wxMenuItem *item, InputMethod how = WithKeyboard);
 
-    // can this tiem be opened?
+    // can this time be opened?
     bool CanOpen(wxMenuItem *item)
     {
         return item && item->IsEnabled() && item->IsSubMenu();
@@ -217,7 +216,7 @@ protected:
     // dismiss the menu and all parent menus too
     void DismissAndNotify();
 
-    // react to dimissing this menu and also dismiss the parent if
+    // react to dismissing this menu and also dismiss the parent if
     // dismissParent
     void HandleDismiss(bool dismissParent);
 
@@ -246,7 +245,7 @@ private:
     // do we currently have an opened submenu?
     bool m_hasOpenSubMenu;
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 // ----------------------------------------------------------------------------
@@ -280,25 +279,27 @@ private:
 // wxWin macros
 // ----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(wxPopupMenuWindow, wxPopupTransientWindow)
+wxBEGIN_EVENT_TABLE(wxPopupMenuWindow, wxPopupTransientWindow)
     EVT_KEY_DOWN(wxPopupMenuWindow::OnKeyDown)
 
+    EVT_LEFT_DOWN(wxPopupMenuWindow::OnLeftDown)
     EVT_LEFT_UP(wxPopupMenuWindow::OnLeftUp)
     EVT_MOTION(wxPopupMenuWindow::OnMouseMove)
     EVT_LEAVE_WINDOW(wxPopupMenuWindow::OnMouseLeave)
+    EVT_MOUSE_CAPTURE_LOST(wxPopupMenuWindow::OnCaptureLost)
 #ifdef __WXMSW__
     EVT_IDLE(wxPopupMenuWindow::OnIdle)
 #endif
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
-BEGIN_EVENT_TABLE(wxMenuBar, wxMenuBarBase)
+wxBEGIN_EVENT_TABLE(wxMenuBar, wxMenuBarBase)
     EVT_KILL_FOCUS(wxMenuBar::OnKillFocus)
-
     EVT_KEY_DOWN(wxMenuBar::OnKeyDown)
-
     EVT_LEFT_DOWN(wxMenuBar::OnLeftDown)
+    EVT_LEFT_UP(wxMenuBar::OnLeftUp)
     EVT_MOTION(wxMenuBar::OnMouseMove)
-END_EVENT_TABLE()
+    EVT_MOUSE_CAPTURE_LOST(wxMenuBar::OnCaptureLost)
+wxEND_EVENT_TABLE()
 
 // ============================================================================
 // implementation
@@ -358,7 +359,7 @@ void wxPopupMenuWindow::ChangeCurrent(wxMenuItemIter node)
             wxCHECK_RET( item, wxT("no current item?") );
 
             // if it was the currently opened menu, close it
-            if ( item->IsSubMenu() && item->GetSubMenu()->IsShown() )
+            if ( node && item->IsSubMenu() && item->GetSubMenu()->IsShown() )
             {
                 item->GetSubMenu()->Dismiss();
                 OnSubmenuDismiss( false );
@@ -368,7 +369,13 @@ void wxPopupMenuWindow::ChangeCurrent(wxMenuItemIter node)
         }
 
         if ( m_nodeCurrent )
-            RefreshItem(m_nodeCurrent->GetData());
+        {
+            wxMenuItem *item = m_nodeCurrent->GetData();
+            if ( item && item->GetMenu()->IsShown() )
+            {
+                RefreshItem(m_nodeCurrent->GetData());
+            }
+        }
     }
 }
 
@@ -474,6 +481,9 @@ void wxPopupMenuWindow::Dismiss()
         OnSubmenuDismiss( false );
     }
 
+    if ( HasCapture() )
+        ReleaseMouse();
+
     wxPopupTransientWindow::Dismiss();
 
     ResetCurrent();
@@ -720,6 +730,18 @@ bool wxPopupMenuWindow::ProcessLeftDown(wxMouseEvent& event)
     return false;
 }
 
+void wxPopupMenuWindow::OnLeftDown(wxMouseEvent& event)
+{
+    wxMenuBar* menubar = m_menu->GetMenuBar();
+    if ( menubar && !GetMenuItemFromPoint(event.GetPosition()) && !ProcessLeftDown(event) )
+    {
+        wxPoint pos = event.GetPosition();
+        wxPoint posScreen = ClientToScreen(pos);
+        event.SetPosition(menubar->ScreenToClient(posScreen));
+        menubar->ProcessEvent(event);
+    }
+}
+
 void wxPopupMenuWindow::OnLeftUp(wxMouseEvent& event)
 {
     wxMenuItemIter node = GetMenuItemFromPoint(event.GetPosition());
@@ -727,6 +749,17 @@ void wxPopupMenuWindow::OnLeftUp(wxMouseEvent& event)
     {
         ActivateItem(node->GetData(), WithMouse);
     }
+    else
+    {
+        wxMenuBar* menubar = m_menu->GetMenuBar();
+        if ( menubar && !ProcessLeftDown(event) )
+        {
+            wxPoint pos = event.GetPosition();
+            wxPoint posScreen = ClientToScreen(pos);
+            event.SetPosition(menubar->ScreenToClient(posScreen));
+            menubar->ProcessEvent(event);
+        }
+    }
 }
 
 void wxPopupMenuWindow::OnMouseMove(wxMouseEvent& event)
@@ -1044,6 +1077,11 @@ bool wxPopupMenuWindow::ProcessKeyDown(int key)
     return processed;
 }
 
+void wxPopupMenuWindow::OnCaptureLost(wxMouseCaptureLostEvent& WXUNUSED(event))
+{
+    DismissAndNotify();
+}
+
 // ----------------------------------------------------------------------------
 // wxMenu
 // ----------------------------------------------------------------------------
@@ -1127,6 +1165,11 @@ wxMenuItem* wxMenu::DoAppend(wxMenuItem *item)
             // for now it has just one element
             item->SetAsRadioGroupStart();
             item->SetRadioGroupEnd(m_startRadioGroup);
+
+            if ( !wxMenuBase::DoAppend(item) )
+                return NULL;
+
+            item->Check(true);
         }
         else // extend the current radio group
         {
@@ -1142,15 +1185,18 @@ wxMenuItem* wxMenu::DoAppend(wxMenuItem *item)
             {
                 wxFAIL_MSG( wxT("where is the radio group start item?") );
             }
+
+            if ( !wxMenuBase::DoAppend(item) )
+                return NULL;
         }
     }
     else // not a radio item
     {
         EndRadioGroup();
-    }
 
-    if ( !wxMenuBase::DoAppend(item) )
-        return NULL;
+        if ( !wxMenuBase::DoAppend(item) )
+            return NULL;
+    }
 
     OnItemAdded(item);
 
@@ -1159,8 +1205,69 @@ wxMenuItem* wxMenu::DoAppend(wxMenuItem *item)
 
 wxMenuItem* wxMenu::DoInsert(size_t pos, wxMenuItem *item)
 {
-    if ( !wxMenuBase::DoInsert(pos, item) )
-        return NULL;
+    if ( item->GetKind() == wxITEM_RADIO )
+    {
+        unsigned int start, end;
+        wxMenuItemIter firstRadio;
+
+        if ( m_startRadioGroup == -1 )
+        {
+            // start a new radio group
+            m_startRadioGroup = pos;
+
+            // set this element as the first of radio group
+            item->SetAsRadioGroupStart();
+            item->SetRadioGroupEnd(m_startRadioGroup);
+
+            if ( !wxMenuBase::DoInsert(pos, item) )
+                return NULL;
+
+            item->Check(true);
+        }
+        else // extend the current radio group
+        {
+            // get current first radio item in radio group
+            firstRadio = GetMenuItems().Item(m_startRadioGroup);
+
+            // get current radio group range
+            start = firstRadio->GetData()->GetRadioGroupStart();
+            end = firstRadio->GetData()->GetRadioGroupEnd();
+
+            if ( pos <= start )
+            {
+                // Item is inserted in the beginning of the range
+                // we need to update its end item
+                m_startRadioGroup = pos;
+                item->SetAsRadioGroupStart();
+                item->SetRadioGroupEnd(end + 1);
+            }
+            else if ( (pos >= start) && (pos <= end) )
+            {
+                // we need to update its end item
+                item->SetRadioGroupStart(m_startRadioGroup);
+
+                // Item is inserted in the middle of this range or immediately
+                // after it in which case it extends this range so make it span
+                // one more item in any case.
+                if ( firstRadio )
+                {
+                    firstRadio->GetData()->SetRadioGroupEnd(end + 1);
+                }
+                else
+                {
+                    wxFAIL_MSG( wxT("where is the radio group start item?") );
+                }
+            }
+
+            if ( !wxMenuBase::DoInsert(pos, item) )
+                return NULL;
+        }
+    }
+    else
+    {
+        if ( !wxMenuBase::DoInsert(pos, item) )
+            return NULL;
+    }
 
     OnItemAdded(item);
 
@@ -1358,7 +1465,7 @@ bool wxMenu::ClickItem(wxMenuItem *item)
     }
     else
     {
-        // not applicabled
+        // not applicable
         isChecked = -1;
     }
 
@@ -1443,8 +1550,6 @@ wxMenuItem::wxMenuItem(wxMenu *parentMenu,
     m_radioGroup.start = -1;
     m_isRadioGroupStart = false;
 
-    m_bmpDisabled = wxNullBitmap;
-
     UpdateAccelInfo();
 }
 
@@ -1508,10 +1613,10 @@ void wxMenuItem::SetCheckable(bool checkable)
     }
 }
 
-void wxMenuItem::SetBitmaps(const wxBitmap& bmpChecked,
-                            const wxBitmap& bmpUnchecked)
+void wxMenuItem::SetBitmaps(const wxBitmapBundle& bmpChecked,
+                            const wxBitmapBundle& bmpUnchecked)
 {
-    m_bmpChecked = bmpChecked;
+    m_bitmap = bmpChecked;
     m_bmpUnchecked = bmpUnchecked;
 
     NotifyMenu();
@@ -1604,6 +1709,16 @@ void wxMenuItem::SetRadioGroupEnd(int end)
     m_radioGroup.end = end;
 }
 
+int wxMenuItem::GetRadioGroupStart()
+{
+    return m_radioGroup.start;
+}
+
+int wxMenuItem::GetRadioGroupEnd()
+{
+    return m_radioGroup.end;
+}
+
 // ----------------------------------------------------------------------------
 // wxMenuBar creation
 // ----------------------------------------------------------------------------
@@ -1692,6 +1807,8 @@ bool wxMenuBar::Insert(size_t pos, wxMenu *menu, const wxString& title)
     if ( !wxMenuBarBase::Insert(pos, menu, title) )
         return false;
 
+    menu->SetTitle( title );
+
     wxMenuInfo *info = new wxMenuInfo(title);
     m_menuInfos.Insert(info, pos);
 
@@ -2027,20 +2144,33 @@ void wxMenuBar::OnLeftDown(wxMouseEvent& event)
     if ( HasCapture() )
     {
         OnDismiss();
-
         event.Skip();
     }
     else // we didn't have mouse capture, capture it now
     {
-        m_current = GetMenuFromPoint(event.GetPosition());
-        if ( m_current == -1 )
+        int item = GetMenuFromPoint(event.GetPosition());
+        if ( item == -1 )
         {
-            // unfortunately, we can't prevent wxMSW from giving us the focus,
-            // so we can only give it back
-            GiveAwayFocus();
+            if ( IsShowingMenu() )
+            {
+                DismissMenu(); // event outside menubar - dismiss
+                ReleaseMouseCapture(); // we could get capture back from popup window so release it
+                RefreshItem((size_t)m_current);
+                m_current = -1;
+            }
+        }
+        else if ( item == m_current && IsShowingMenu() )
+        {
+            // double-click
+            DismissMenu();
+            ReleaseMouseCapture();
+            RefreshItem((size_t)m_current);
+            m_current = -1;
         }
         else // on item
         {
+            m_current = item;
+
             wxLogTrace(wxT("mousecapture"), wxT("Capturing mouse from wxMenuBar::OnLeftDown"));
             CaptureMouse();
 
@@ -2053,6 +2183,17 @@ void wxMenuBar::OnLeftDown(wxMouseEvent& event)
     }
 }
 
+void wxMenuBar::OnLeftUp(wxMouseEvent& event)
+{
+    if ( !HasCapture() && IsShowingMenu() && GetMenuFromPoint(event.GetPosition()) == -1 )
+    {
+        DismissMenu();
+        ReleaseMouseCapture();
+        RefreshItem((size_t)m_current);
+        m_current = -1;
+    }
+}
+
 void wxMenuBar::OnMouseMove(wxMouseEvent& event)
 {
     if ( HasCapture() )
@@ -2236,6 +2377,14 @@ void wxMenuBar::OnKeyDown(wxKeyEvent& event)
     }
 }
 
+void wxMenuBar::OnCaptureLost(wxMouseCaptureLostEvent& WXUNUSED(event))
+{
+    if ( IsShowingMenu() )
+    {
+        DismissMenu();
+    }
+}
+
 // ----------------------------------------------------------------------------
 // wxMenuBar accel handling
 // ----------------------------------------------------------------------------
@@ -2412,6 +2561,7 @@ void wxMenuBar::OnDismiss()
         m_current = -1;
 
         RefreshItem(current);
+        Update();
     }
 
     GiveAwayFocus();
