@@ -5,7 +5,7 @@
 // Modified by: Robin Dunn
 //
 // Created:     25-May-2006
-// Copyright:   (c) 2006-2018 Total Control Software
+// Copyright:   (c) 2006-2020 Total Control Software
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -141,12 +141,13 @@ pdcDrawPolyPolygonOp::pdcDrawPolyPolygonOp(int n, int count[], wxPoint points[],
 
 pdcDrawPolyPolygonOp::~pdcDrawPolyPolygonOp()
 {
-    if (m_points) delete m_points;
-    if (m_count) delete m_count;
+    if (m_points) delete[] m_points;
+    if (m_count) delete[] m_count;
     m_points=NULL;
     m_count=NULL;
 }
 
+
 // ----------------------------------------------------------------------------
 // pdcDrawLinesOp
 // ----------------------------------------------------------------------------
@@ -482,7 +483,7 @@ bool wxPseudoDC::GetIdGreyedOut(int id)
 // ----------------------------------------------------------------------------
 PyObject *wxPseudoDC::FindObjectsByBBox(wxCoord x, wxCoord y)
 {
-    //wxPyBlock_t blocked = wxPyBeginBlockThreads();
+    wxPyThreadBlocker blocker;
     pdcObjectList::compatibility_iterator pt = m_objectlist.GetFirst();
     pdcObject *obj;
     PyObject* pyList = NULL;
@@ -500,7 +501,6 @@ PyObject *wxPseudoDC::FindObjectsByBBox(wxCoord x, wxCoord y)
         }
         pt = pt->GetNext();
     }
-    //wxPyEndBlockThreads(blocked);
     return pyList;
 }
 
@@ -510,7 +510,7 @@ PyObject *wxPseudoDC::FindObjectsByBBox(wxCoord x, wxCoord y)
 PyObject *wxPseudoDC::FindObjects(wxCoord x, wxCoord y,
                                   wxCoord radius, const wxColor& bg)
 {
-    //wxPyBlock_t blocked = wxPyBeginBlockThreads();
+    wxPyThreadBlocker blocker;
     pdcObjectList::compatibility_iterator pt = m_objectlist.GetFirst();
     pdcObject *obj;
     PyObject* pyList = NULL;
@@ -610,8 +610,7 @@ PyObject *wxPseudoDC::FindObjects(wxCoord x, wxCoord y,
         maskdc.SelectObject(wxNullBitmap);
         memdc.SelectObject(wxNullBitmap);
     }
-    //wxPyEndBlockThreads(blocked);
-    return pyList;
+     return pyList;
 }
 
 // ----------------------------------------------------------------------------
