@@ -12,7 +12,7 @@ class bannerwindow_Tests(wtc.WidgetTestCase):
 
     def test_bannerwindow1(self):
         banner = wx.adv.BannerWindow(self.frame, dir=wx.LEFT)
-        banner.SetBitmap(wx.Bitmap(pngFile))
+        banner.SetBitmap(wx.BitmapBundle(wx.Bitmap(pngFile)))
 
     def test_bannerwindow2(self):
         banner = wx.adv.BannerWindow(self.frame, dir=wx.LEFT)
