@@ -14,6 +14,7 @@ imgFiles = [os.path.join(os.path.dirname(__file__), 'LB01.png'),
 
 class bmpcbox_Tests(wtc.WidgetTestCase):
 
+    @unittest.skipIf('wxMac' in wx.PlatformInfo, 'Needs a real MainLoop on wxMac')
     def test_bmpcbox1(self):
         pnl = wx.Panel(self.frame)
 
@@ -31,7 +32,10 @@ class bmpcbox_Tests(wtc.WidgetTestCase):
         bcb.SetClientData(2, "Bye")
         self.assertEqual(bcb.GetClientData(2), "Bye")
 
+        self.waitFor(300)
 
+
+    @unittest.skipIf('wxMac' in wx.PlatformInfo, 'Needs a real MainLoop on wxMac')
     def test_bmpcbox2(self):
         pnl = wx.Panel(self.frame)
 
@@ -42,6 +46,8 @@ class bmpcbox_Tests(wtc.WidgetTestCase):
 
         self.assertEqual(bcb.GetClientData(2), "2")
 
+        self.waitFor(300)
+
 
 #---------------------------------------------------------------------------
 
